package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.container.StreamProbeResult;
import net.knarcraft.ffmpegconverter.converter.module.ConverterModule;
import net.knarcraft.ffmpegconverter.converter.module.DebugModule;
import net.knarcraft.ffmpegconverter.converter.module.ModuleExecutor;
import net.knarcraft.ffmpegconverter.converter.module.mapping.NthAudioStreamModule;
import net.knarcraft.ffmpegconverter.converter.module.output.SetOutputFileModule;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;

/**
 * An extractor for getting audio streams from video files
 */
public class AudioExtractor extends AbstractConverter {

    private final int streamToExtract;

    /**
     * Instantiates a new audio extractor
     *
     * @param ffprobePath     <p>Path/command to ffprobe.</p>
     * @param ffmpegPath      <p>Path/command to ffmpeg.</p>
     * @param newExtension    <p>The extension of the new file.</p>
     * @param streamToExtract <p>The stream to be extracted from the video file</p>
     */
    public AudioExtractor(@NotNull String ffprobePath, @NotNull String ffmpegPath, @NotNull String newExtension,
                          int streamToExtract) {
        super(newExtension);
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
        this.streamToExtract = Math.max(0, streamToExtract);
    }

    @Override
    @Nullable
    public FFMpegCommand generateConversionCommand(@NotNull String executable, @NotNull StreamProbeResult probeResult,
                                                   @NotNull String outFile) {
        FFMpegCommand command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, probeResult.parsedFiles());
        List<ConverterModule> modules = new ArrayList<>();

        if (this.debug) {
            modules.add(new DebugModule());
        }

        //Gets the first audio stream from the file and adds it to the output file
        modules.add(new NthAudioStreamModule(probeResult.getAudioStreams(), streamToExtract));
        modules.add(new SetOutputFileModule(outFile));

        new ModuleExecutor(command, modules).execute();
        return command;
    }

    @Override
    @NotNull
    public List<String> getValidFormats() {
        return videoFormats;
    }

}
