package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.container.StreamProbeResult;
import net.knarcraft.ffmpegconverter.converter.module.ConverterModule;
import net.knarcraft.ffmpegconverter.converter.module.DebugModule;
import net.knarcraft.ffmpegconverter.converter.module.ModuleExecutor;
import net.knarcraft.ffmpegconverter.converter.module.mapping.MapAllModule;
import net.knarcraft.ffmpegconverter.converter.module.output.CopySubtitlesModule;
import net.knarcraft.ffmpegconverter.converter.module.output.CopyVideoModule;
import net.knarcraft.ffmpegconverter.converter.module.output.SetOutputFileModule;
import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.streams.VideoStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;

/**
 * A converter to convert a video's audio into flac
 */
public class AudioToVorbisConverter extends AbstractConverter {

    /**
     * Initializes variables used by the abstract converter
     *
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath  <p>Path/command to ffmpeg.</p>
     */
    public AudioToVorbisConverter(String ffprobePath, String ffmpegPath) {
        super("mkv");
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
    }

    @Override
    public @NotNull List<String> getValidFormats() {
        return this.videoFormats;
    }

    @Override
    public @Nullable FFMpegCommand generateConversionCommand(@NotNull String executable, @NotNull StreamProbeResult probeResult, @NotNull String outFile) {
        FFMpegCommand command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, probeResult.parsedFiles());
        List<ConverterModule> modules = new ArrayList<>();

        if (this.debug) {
            modules.add(new DebugModule());
        }

        // Map video if present
        List<VideoStream> videoStreams = probeResult.getVideoStreams();
        if (!videoStreams.isEmpty()) {
            modules.add(new MapAllModule<>(videoStreams));
            modules.add(new CopyVideoModule());
        }

        // Map audio if present
        List<AudioStream> audioStreams = probeResult.getAudioStreams();
        if (!audioStreams.isEmpty()) {
            modules.add(new MapAllModule<>(audioStreams));
            setOutputIndexes(audioStreams);
        }

        // Map subtitles if present
        List<SubtitleStream> subtitleStreams = probeResult.getSubtitleStreams();
        if (!subtitleStreams.isEmpty()) {
            modules.add(new MapAllModule<>(subtitleStreams));
            setOutputIndexes(subtitleStreams);
            modules.add(new CopySubtitlesModule());
        }

        // Map any fonts, cover images or similar
        modules.add(new MapAllModule<>(probeResult.getOtherStreams()));

        // Set output file and execute
        modules.add(new SetOutputFileModule(outFile));
        new ModuleExecutor(command, modules).execute();
        return command;
    }

}
