package net.knarcraft.ffmpegconverter.converter.module.mapping;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.converter.module.ConverterModule;
import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import org.jetbrains.annotations.NotNull;

/**
 * A module for adding an audio stream, and converting it to stereo
 */
public class AddStereoAudioStreamModule implements ConverterModule {

    private final AudioStream audioStream;
    private final boolean mapAudio;

    /**
     * Instantiates a new add stereo audio stream module
     *
     * @param audioStream <p>The audio stream to add and convert to stereo</p>
     * @param mapAudio    <p>Whether to map the given audio stream (only disable if mapped elsewhere)</p>
     */
    public AddStereoAudioStreamModule(@NotNull AudioStream audioStream, boolean mapAudio) {
        this.audioStream = audioStream;
        this.mapAudio = mapAudio;
    }

    @Override
    public void addArguments(@NotNull FFMpegCommand command) {
        if (mapAudio) {
            FFMpegHelper.mapStream(command, audioStream);
        }
        if (audioStream.getChannels() > 2) {
            command.addOutputFileOption("-af", "pan=stereo|FL=FC+0.30*FL+0.30*BL|FR=FC+0.30*FR+0.30*BR");
        }
    }

}
