package net.knarcraft.ffmpegconverter.converter.module.mapping;

import net.knarcraft.ffmpegconverter.container.FFMpegCommand;
import net.knarcraft.ffmpegconverter.converter.module.ConverterModule;
import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;
import org.jetbrains.annotations.NotNull;

import java.util.List;

/**
 * A module for selecting and mapping the nth audio stream available
 */
public class NthAudioStreamModule implements ConverterModule {

    private final int n;
    private final List<AudioStream> allStreams;

    /**
     * Instantiates a new n-th audio stream module
     *
     * @param allStreams <p>All available streams</p>
     * @param n          <p>The index of the stream the user wants</p>
     */
    public NthAudioStreamModule(@NotNull List<AudioStream> allStreams, int n) {
        this.allStreams = allStreams;
        this.n = n;
    }

    @Override
    public void addArguments(@NotNull FFMpegCommand command) {
        AudioStream audioStream = FFMpegHelper.getNthSteam(this.allStreams, this.n);
        if (audioStream != null) {
            FFMpegHelper.mapStream(command, audioStream);
        } else {
            throw new IllegalArgumentException("Selected audio stream does not exist.");
        }
    }

}
