package net.knarcraft.ffmpegconverter.streams;

import net.knarcraft.ffmpegconverter.utility.ValueParsingHelper;
import org.jetbrains.annotations.NotNull;

import java.util.Map;

/**
 * An object representation of a video stream in a media file
 */
public class VideoStream extends AbstractStream implements StreamObject {

    private final int width;
    private final int height;

    /**
     * Instantiates a new video stream
     *
     * @param streamInfo    <p>All info about the stream</p>
     * @param inputIndex    <p>The index of the input file containing this stream</p>
     * @param relativeIndex <p>The index of the video stream relative to other video streams</p>
     */
    public VideoStream(@NotNull Map<StreamTag, String> streamInfo, int inputIndex, int relativeIndex) {
        super(streamInfo, inputIndex, relativeIndex);
        this.width = ValueParsingHelper.parseInt(streamInfo.get(StreamTag.WIDTH), -1);
        this.height = ValueParsingHelper.parseInt(streamInfo.get(StreamTag.HEIGHT), -1);
    }

    /**
     * Gets the width of the video stream
     *
     * @return <p>The width of the video steam in pixels.</p>
     */
    public int getWidth() {
        return this.width;
    }

    /**
     * Gets the height of the video stream
     *
     * @return <p>The height of the video stream in pixels.</p>
     */
    public int getHeight() {
        return this.height;
    }

    @Override
    public char streamTypeCharacter() {
        return 'v';
    }

}